CATEGORY_TREE = {
    'food-and-drink': [
        'restaurant',
        'food-court',
        'fast-food',
        'coffee-shop',
        'pub',
    ],
    'healthcare': [
        'hospital',
        'clinic',
        'doctor',
        'pharmacy',
    ],
    'education': [
        'school',
        'college',
        'university',
        'kindergarten',
    ],
    'public-transport': [
        'bus-stop',
        'railway-station',
    ],
    'shopping': [
        'supermarket',
        'mall',
        'convenience-store',
    ],
    'green-spaces': [
        'park',
        'garden',
        'nature-reserve',
    ],
    'landform': [
        'beach',
    ],
    'fitness': [
        'gym',
        'fitness-station',
        'sports-centre',
    ],
}



from typing import Dict, List

import pytest


@pytest.fixture
def properties() -> List[Dict[str, str]]:
    return [
        {
            "id": "Property A",
            "latitude": "51.0",
            "longitude": "0.0"
        },
        {
            "id": "Property B",
            "latitude": "51.1",
            "longitude": "0.1"
        },
        {
            "id": "Property C",
            "latitude": "51.2",
            "longitude": "0.2"
        }
    ]


@pytest.fixture
def needs() -> Dict[str, Dict]:
    return {
        "food-and-drink": {
            "importance": 5,
            "categories": [
                "fast-food"
            ]
        }
    }


@pytest.fixture
def destinations() -> List[Dict]:
    return [
        {
            "id": "destination",
            "latitude": 25.197197,
            "longitude": 55.27437639999999,
            "time": 10,
            "methods": [
                "walking", "driving"
            ]
        }
    ]



from unittest.mock import patch

from propertia.client import PropertiaClient


@patch(
    target='propertia.client.PropertiaClient.make_post_call',
    return_value={
        "results": [
            {
                "id": "Property C",
                "latitude": "51.2",
                "longitude": "0.2",
                "smartscore": 10,
                "scores": {
                    "food-and-drink": 10
                }
            },
            {
                "id": "Property B",
                "latitude": "51.1",
                "longitude": "0.1",
                "smartscore": 8,
                "scores": {
                    "food-and-drink": 8
                }
            },
            {
                "id": "Property A",
                "latitude": "51.0",
                "longitude": "0.0",
                "smartscore": 6,
                "scores": {
                    "food-and-drink": 6
                }
            }
        ]
    }
)
def test_get_scores(properties, needs):
    with PropertiaClient(api_key="dummy") as client:
        scores = client.get_scores(properties, needs)
        assert len(scores["results"]) == 3
        assert scores["results"][0]["id"] == "Property C"
        assert scores["results"][1]["id"] == "Property B"
        assert scores["results"][2]["id"] == "Property A"



from unittest.mock import patch

from propertia.client import PropertiaClient


@patch(
    target='propertia.client.PropertiaClient.make_post_call',
    return_value={
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "geometry": {
                    "type": "MultiPolygon",
                    "coordinates": ["some_coordinates_for_you"]
                },
            }
        ]
    }
)
def test_get_isochrones(destinations):
    with PropertiaClient(api_key="dummy") as client:
        polygon = client.get_isochrones(destinations, True)
        assert polygon["type"] == "FeatureCollection"



from typing import Any, Dict, List

import httpx


class PropertiaClient:
    SMARTSCORE_ENDPOINT = '/smartscore/'
    ISOCHRONES_ENDPOINT = '/isochrones/'
    TRAVEL_TIME_ENDPOINT = '/travel-time/'
    USER_CATEGORIES_ENDPOINT = '/user-categories/'

    def __init__(self, api_key: str, host: str = "https://propertia.searchsmartly.co") -> None:
        self._host = host.rstrip("/")
        self._api_key = api_key
        self._headers = {
            "Authorization": f"Bearer {self._api_key}",
            "Content-Type": "application/json",
        }
        self.client = httpx.Client(
            transport=httpx.HTTPTransport(retries=3),
            base_url=self._host,
            headers=self._headers,
            timeout=httpx.Timeout(connect=None, read=None, write=None, pool=None)
        )

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.client.close()

    def get_scores(self, properties: List, needs: Dict[str, Any]) -> Dict[str, List]:
        payload = {
            "needs": needs,
            "properties": properties,
        }
        return self.make_post_call(self.SMARTSCORE_ENDPOINT, payload)

    def get_isochrones(self, destinations: List, aggregated: bool) -> Dict[str, Any]:
        """
        The destinations parameter is a list of 1 or 2 dictionaries, each of which must contain the following:
        {
            "id": "destination",
            "latitude": 25.197197,
            "longitude": 55.27437639999999,
            "time": 10,
            "methods": [
                "walking", "driving", "cycling", "public_transport"
            ]
        }
        The aggregated parameter is a boolean that specifies whether to return a single isochrone or a list of them
        """
        payload = {
            "destinations": destinations,
            "aggregated": aggregated
        }
        return self.make_post_call(self.ISOCHRONES_ENDPOINT, payload)

    def get_travel_time(self, destinations: List, properties: List) -> Dict[str, List]:
        """
        The destinations parameter is a list of 1 or 2 dictionaries, each of which must contain the following:
        {
            "id": "destination",
            "latitude": 25.197197,
            "longitude": 55.27437639999999,
            "time": 10,
            "methods": [
                "walking", "driving", "cycling", "public_transport"
            ]
        }
        The properties parameter is a list of dictionaries, each of which must contain the following:
        {
            "id": "string",
            "latitude": -90,
            "longitude": -180
        }
        """
        payload = {
            "origin": destinations,
            "properties": properties,
        }
        return self.make_post_call(self.TRAVEL_TIME_ENDPOINT, payload)

    def make_post_call(self, endpoint: str, payload: Dict) -> Dict[str, List]:
        return self.client.post(endpoint, json=payload).json()

    def get_user_categories(self) -> Dict[str, List]:
        return self.make_get_call(self.USER_CATEGORIES_ENDPOINT)

    def make_get_call(self, endpoint: str) -> Dict[str, List]:
        return self.client.get(endpoint).json()



from unittest.mock import patch

from propertia.client import PropertiaClient


@patch(
    target='propertia.client.PropertiaClient.make_post_call',
    return_value={
        "Property A": {
            "walking": {
                "origin_method_time": 10
            }
        },
        "Property B": {
            "walking": {
                "origin_method_time": 12
            }
        },
        "Property C": {
            "walking": {
                "origin_method_time": 16
            }
        }
    }
)
def test_get_travel_time(destinations, properties):
    with PropertiaClient(api_key="dummy") as client:
        results = client.get_travel_time(destinations, properties)
        assert len(results) == 3
